import redditcleaner #Not in standard lib.
import time
import requests
import traceback
import os,sys
import imghdr
from datetime import datetime

from zipfile import ZipFile
import json
from pprint import pprint
from pathlib import Path
from rich.table import Table,Column
from rich.console import Console

#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def cleanupPostData(postData, postType): #Thanks /u/kwelzel for helping me refactor!
    interestingPostData = {}
    submissionAndCommentKeys = ('id', 'created_utc', 'subreddit', 'score', 'link_id', 'parent_id')

    for i in submissionAndCommentKeys:
        if not i in postData.keys():
            interestingPostData[i] = None

        else:
            interestingPostData[i] = postData[i]

    timestamp = int(postData['created_utc'])
    interestingPostData['datetime'] = str(datetime.utcfromtimestamp(timestamp).strftime('%a %b %d %Y, %I:%M %p UTC'))

    if postType == 'comment':
        if 'body' in postData.keys():
            interestingPostData['body'] = textPostWords(postData['body'])
        else:
            interestingPostData['body'] = None
        if 'permalink' in postData.keys():
            interestingPostData['permalink'] = f'https://www.reddit.com{postData["permalink"]}'
        else:
            interestingPostData['permalink'] = None

    if postType == 'submission':
        if 'selftext' in postData.keys():
            interestingPostData['selftext'] = textPostWords(postData['selftext'])
        else:
            interestingPostData['selftext'] = None
        if 'url' in postData.keys():
            interestingPostData['url'] = postData["url"]
        else:
            interestingPostData['url'] = None
        if 'full_link' in postData.keys():
            interestingPostData['full_link'] = postData["full_link"]
        else:
            interestingPostData['full_link'] = None

    return interestingPostData
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def pushshift(lim, postType, postSetMaxLen, user='', log=False): #Pushshift requests
    console = Console()
    before = int(round(time.time()))

    if postType == 'comment':
        highlight = '[cyan]'
    else:
        highlight = '[magenta]'

    ct = 0

    while True:

        if lim is not None:
            if ct >= lim:
                break

        apiUrl = 'https://api.pushshift.io/reddit/search/'
        url = f'{apiUrl}{postType}/?author={user}&size={postSetMaxLen}&before={before}'

        response = requests.get(apiUrl + postType, params={'author': user, 'size': postSetMaxLen, 'before': before})
        data = response.json()['data']
        if not data:
            break

        for postData in data:
            before = postData['created_utc']
            yield cleanupPostData(postData, postType)

        ct += len(data)

        if log:
            console.print(f'[bold green]\t{ct} {highlight}{url}')

        if len(data) < postSetMaxLen:
            break

        time.sleep(.75)
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def countRequests(lim, postSetMaxLen):
    postSetMaxLen = 100
    numOfRequests = lim // postSetMaxLen
    if lim % postSetMaxLen != 0:
        remainingPosts = lim % postSetMaxLen
        numOfRequests +=1

    return numOfRequests
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def comments(lim=None, user='', log=False):
    postSetMaxLen = 100
    comment = [i for i in pushshift(lim, 'comment', postSetMaxLen, user=user, log=log)]
    return comment[:lim]
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def submissions(lim=None, user='', log=False):
    postSetMaxLen = 100
    submission = [i for i in pushshift(lim, 'submission', postSetMaxLen, user=user, log=log)]
    return submission[:lim]
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def textPostWords(redditRawText): # Makes body and selftext not an abomination.
    return redditcleaner.clean(redditRawText).split()
    '''
    # Spliting post string into sets of 15 words so the output is readable when it reaches it's place within json.
    splitWords = []
    temp = []
    for i, word in enumerate(cleaned):
        temp.append(word)
        if i % 15 == 0 and i != 0:
            splitWords.append(temp)
            temp = []

    # Another way of saying if the number of totalW words % 15 != 0. Need to put the leftover words where they belong
    if len(temp) != 0:
        splitWords.append(temp)

    return [' '.join(cleanPost) for cleanPost in splitWords]
    '''
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def countPosts(allPosts):  # Count and order most posted subs.
    postCounts = {}
    for postType, posts in allPosts.items():
        subreddits = [post['subreddit'] for post in posts if 'subreddit' in post.keys()]
        subredditSet = set(subreddits)

        counts = []
        for subreddit in subredditSet:
            if subreddit is not None:
                counts.append([subreddit,subreddits.count(subreddit)])

        #Sort by number of posts user has in each sub that they have posted in.
        sortedCounts = sorted(counts, key=lambda subredditPostCount: (subredditPostCount[1], subredditPostCount[0]), reverse=True)
        postCounts[postType] = sortedCounts

    return postCounts
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def writeFiles(allPosts, postCounts, user, userDir):
    if len(allPosts) != 0:
        jPath = os.path.join(userDir, 'all_posts.json')
        with open(jPath, 'w+', newline='\n') as f:
            json.dump(allPosts, f, indent=4)

        tPath = os.path.join(userDir, 'subreddit_count.txt')
        with open(tPath, 'w+') as g:
            for k, v in postCounts.items():
                g.write(f'{k}\n')
                for i in v:
                    g.write(f'{i[0]}, {str(i[1])}\n')

#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def imageUrls(user, submissions):
    urls = [v for i in submissions  for k,v in i.items() if k == 'url']
    imageUrls = []
    for url in urls:
        if url.endswith(('.png', '.PNG', '.jpg', '.JPG', '.gif', '.GIF')):
            imageUrls.append(url)
            continue

        if '://reddit.com' in url:
            continue
        if 'imgur.com' in url:
            if '/a/' in url:
                imageUrls.append(url + '/zip')
            else:
                imageUrls.append(url + '.jpg')

    imageUrlsStr = '\n'.join(imageUrls)
    userDir = Path.cwd() / 'users' / user
    images = os.path.join(str(userDir), 'images.txt')
    with open(images, 'w+') as f:
        f.write(imageUrlsStr+ '\n')
    return imageUrlsStr
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def imagesdl(images, userDir): #Needs to deal with all the nonsense involved with image formatting
    console = Console()
    for i, url in enumerate(images):
        try:
            response = requests.get(url, stream=True)

            if url.endswith(('.png', '.PNG', '.jpg', '.JPG', '.gif', '.GIF')):
                urlType = 'image'
                fileType = f'{url.split(".")[-1]}'
                imagePath = os.path.join(userDir, f'{str(i+1)}.{fileType}')

            else:
                urlType = 'zip'
                fileType = 'urlType'
                imagePath = os.path.join(userDir, f'{i+1}.zip')

            with open(imagePath, 'wb+') as f:
                f.write(response.content)

            dlLog = f'\t{os.path.split(imagePath)[-1]} - Downloaded: {url}'
            console.print(f'[green]{dlLog}')

            try:
                bytes = open(imagePath,'rb').read().decode()[1:9]
                if type(bytes) == str and  bytes[0:2] == 'PK': #A zip file
                    continue

                else:
                    console.print(f'[bold red]Invalid image link - removing {os.path.split(imagePath)[-1]}{" " * 4}')
                    os.remove(imagePath)
                    continue
            except UnicodeDecodeError as e:
                pass

            changedFlag = False
            imghdrExtension = imghdr.what(imagePath)
            if str(imghdrExtension) == 'None':
                imghdrExtension = 'jpeg'
                changedFlag = True

            if urlType == 'image' and not changedFlag:
                newFname = f'{i+1}.{imghdrExtension}'

                newFpath = os.path.join(userDir, newFname)
                os.rename(imagePath, newFpath)

            try:
                with ZipFile(imagePath, 'r') as zipObj:
                    listOfiles = zipObj.namelist()

            except Exception as e:
                if urlType == 'image':
                    continue

                newFname = f'{i+1}.{imghdrExtension}'

                newFpath = os.path.join(userDir, newFname)
                os.rename(imagePath, newFpath)

                logSplit = dlLog.split(' ')
                logSplit[1] = f'{i+1}.{imghdrExtension}'

                changeExtension = str(i+1) + '.zip => ' + newFname

                spaces = len(logSplit[0] + logSplit[1]) - len(changeExtension) + 4
                console.print(f'[bold blue]{changeExtension}{" " * spaces}{e}')

        except Exception as e:
            console.print(f'[bold red]{i} {url} unexpected exception: {traceback.format_exc()}')



#!/usr/bin/env python3

'''
              _     _ _ _        __ _           _
 _ __ ___  __| | __| (_) |_ ___ / _(_)_ __   __| | ___ _ __
| '__/ _ \/ _` |/ _` | | __/ __| |_| | '_ \ / _` |/ _ \ '__|
| | |  __/ (_| | (_| | | |_\__ \  _| | | | | (_| |  __/ |
|_|  \___|\__,_|\__,_|_|\__|___/_| |_|_| |_|\__,_|\___|_|


github.com/fitzy1293/redditsfinder


TO DO:
    - Add more optional args.
    - Instead of using a set to identify pictures:
        - name_com_extension
            -What if invalid extension?
'''

import urllib.request
import requests
import json
import time
import os,sys
import traceback
from pprint import pprint
from urllib.error import HTTPError
from datetime import datetime
from pathlib import Path
import argparse


from rich.table import Table,Column
from rich.console import Console

from .redditsfinder_utils import *
from .after_run_parsing import *

#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def getPosts(user, postType): # Pushshift API requests in chunks of 100
    console = Console()
    console.print(f'[bold blue underline]{postType[0].upper()}{postType[1:]} request log:')

    postSetMaxLen = 100 # Max num of posts in each pushshift request, seems to be 100 right now or it breaks.

    return pushshift(None, postType, postSetMaxLen, user=user, log=True)
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def printTotals(tablesDict): #Prints table after the pushshift log.
    print('\n\n')
    console = Console()

    header = [
            '',
            'Recent comments:\nsubmission ID\ncomment ID',
            'Recent submissions:\nsubmission ID',
            'Comments count:\nfor each sub',
            f'[bold red]{tablesDict["commentsLen"]}[/bold red]',
            'Submissions count:\nfor each sub',
            f'[bold red]{tablesDict["submissionsLen"]}[/bold red]'#,
            #tablesDict["dir"],
            #'Run Time'
    ]

    console.log(f'[magenta]{tablesDict["user"]}')

    table = Table(show_header=True,header_style="bold blue")

    for i in header:
        table.add_column(i, justify='left')

    commentsColumn = '\n'.join([sub[0] for sub in tablesDict['postCounts']['comments']])
    commentsCt = '\n'.join([f'[bold red]{sub[1]}[/bold red]' for sub in tablesDict['postCounts']['comments']])

    submissionsColumn = '\n'.join([sub[0] for sub in tablesDict['postCounts']['submissions']])
    submissionsCt = '\n'.join([f'[bold red]{sub[1]}[/bold red]' for sub in tablesDict['postCounts']['submissions']])

    maxRecentPosts = 50
    posts = postRunJson(os.path.join(tablesDict['dir'], 'all_posts.json'), maxRecentPosts)

    #Numbers the columns
    columnLengths = max (i.count('\n') for i in [posts[0],posts[1], commentsColumn, submissionsColumn]) + 1
    columnNums = '\n'.join([str(i) for i in range(columnLengths)])
    print(columnLengths)


    rows = [
            columnNums,
            posts[0],
            posts[1],
            f'[purple]{commentsColumn}[/purple]',
            commentsCt,
            f'[purple]{submissionsColumn}[/purple]',
            submissionsCt,
            '[magenta underline]all_posts.json\nsubreddit_count.txt',
            f'[magenta underline]{round(tablesDict["end"] - tablesDict["start"], 1)} s'
    ]


    table.add_row(
    rows[0], rows[1], rows[2], rows[3], rows[4], rows[5], rows[6]#, rows[7], rows[8]
    )

    console.print(table, justify='center', style='bold white')
    print()
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def run(cliArgs=[], user='', **kwargs):
    start = time.time()
    console = Console()

    if kwargs:
        cliArgs = ['--' + i for i in kwargs.keys()]

    if 'user' in kwargs.keys():
        user = kwargs['user']


    userDir = Path.cwd() / 'users' / user
    userDir.mkdir(parents=True, exist_ok=True)

    console.print(f'[bold blue]\nGathering and formatting data from pushshift for {user}.\n')

    if  '-pics' in cliArgs or '--pics' in cliArgs:
        keyType = {'submission': ('url', 'created_utc',)}
        images = imageUrls(user, [i for i in getPosts(user, 'submission')])

        images = set(open(os.path.join(userDir,'images.txt')).read().splitlines())
        imageStatus = '\n'.join([f'\t[bold green]{i+1} [cyan]{image}' for i, image in enumerate(images)])
        imageSubmissionLog = f'[bold blue underline]\nImages submitted by {user}:[/bold blue underline]\n{imageStatus}'
        console.print(imageSubmissionLog)

        if '-d' in cliArgs or '--download' in cliArgs:
            print()
            imagesdl(images, userDir)

    else:
        allPosts = {'comments': [i for i in getPosts(user, 'comment')],
                    'submissions': [i for i in getPosts(user, 'submission')]}
        postCounts = countPosts(allPosts)
        writeFiles(allPosts, postCounts, user, userDir)

        if not '-q' in cliArgs and not '--quiet' in cliArgs:
            tablesDict = {'postCounts': postCounts,
                          'commentsLen': str(len(allPosts['comments'])),
                          'submissionsLen': str(len(allPosts['submissions'])),
                          'dir': str(userDir),
                          'start': start,
                          'end': time.time(),
                          'user': user}
            printTotals(tablesDict)

    if not '-q' in cliArgs and not '--quiet' in cliArgs:
        fnamesStr = '\n\t' + '\n\t'.join([i for i in os.listdir(userDir)])
        console.print(f'\n[bold cyan underline]{userDir}{fnamesStr}')
        console.print(f'\n[bold blue]Run time - {round(time.time() - start, 1)} s\n')
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
#=============================================================================================================================
#─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────
def main():  # System arguments

    if len(sys.argv) == 1:
        print('''Running redditsfinder

        Test it on yourself to make sure it works.

        redditsfinder yourusername

        Basic usage

        redditsfinder username
        redditsfinder [options] username_0 username_1 username_2 ...

        With a file

        -f or --file.
        redditsfinder [options] -f line_separated_text_file.txt

        Optional args

        -pics returns URLs of image uploads.
        -pics -d or -pics --download downloads them.
        -q or --quiet turns off non log related print statements.''' )


    else:
        redditsfinderArgs = sys.argv[1:]
        optionalArgs = ('-pics', '-d', '--download', '-q', '--quiet', '-f', '--file')
        enteredOptionalArgs = [i for i in redditsfinderArgs if i in optionalArgs]

        if '-f' in enteredOptionalArgs:
            usernames = open(sys.argv[-1]).read().splitlines()
        else:
            usernames = [i for i in redditsfinderArgs if i not in optionalArgs]

        print(f'Optional arguments: {enteredOptionalArgs}')
        print('Usernames: '), pprint(usernames)

        for user in usernames:
            run(enteredOptionalArgs, user)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['redditsfinder']

package_data = \
{'': ['*']}

install_requires = \
['redditcleaner>=1.1.2,<2.0.0', 'requests>=2.24.0,<3.0.0', 'rich>=7.0.0,<8.0.0']

entry_points = \
{'console_scripts': ['redditsfinder = redditsfinder:main']}

setup_kwargs = {
    'name': 'redditsfinder',
    'version': '1.3.1',
    'description': "Archive a reddit user's post history. Formatted overview of a profile, JSON containing every post, and picture downloads.",
    'long_description': "# redditsfinder --- reddit user info\n**`pip3 install redditsfinder`**\n\n**A command line program to easily download reddit users' post histories.**\n\nGet any reddit user's entire post history with one command while avoiding the reddit API's 1000 post limit. \\\nThe main meat of this program is making the requests to pushshift and manipulating pushshift's JSON for a more readable all_posts.json file. \\\nThere is also a handly image downloader I made that avoids a lot of the problems of trying to grab multiple images from different sites at once. Things like file types being not what the file is encoded as, and changed URLs. Or a URL that ends with .png that returns ASCII text. It gets imgur albums along with images, because at least for a while imgur was essentially reddit's non-official image hosting service.\n\nThe colored terminal features and markup are from https://github.com/willmcgugan/rich \\\n`pip3 install rich` which is one the coolest python packages I've seen. It's very easy to pick up, but as is shown with the animated example in its README, still has a lot of depth.  \n\nhttps://github.com/LoLei/redditcleaner `pip3 install redditcleaner` was also a massive help for dealing with reddit's strange markup. \\\nComments and self-posts can be unreadable when put in another format like JSON if they have a fair amount of formatting. \\\nTo deal with it, I gave up and looked online for an alternative. Luckily there was a good one readily available.\n\n# Installation\n`pip3 install redditsfinder`\n\n# Running redditsfinder\n\n***Test it on yourself to make sure it works.***\n\n`redditsfinder yourusername`\n\n***Basic usage***\n\n**Returns every post to a different JSON file for each user and formats a table in the terminal for a quick view.\\\nTakes an arbitrary number of user names, such that there is at least one user name.**\\\n\\\n`redditsfinder username`\\\n`redditsfinder [options] username_0 username_1 username_2 ...`\n\n\n\n***Newline separated file***\n\n**Uses user names from a file.**\\\n\\\n`-f` or `--file`\\\n`redditsfinder [options] -f line_separated_text_file.txt`\n\n\n\n\n***Optional args***\n\n`-pics` returns URLs of image uploads.\\\n`-pics -d` or `-pics --download` downloads them.\\\n`-q` or `--quiet` turns off non log related print statements.\n\n# Example Pushshift request log\n![Imgur Image](https://imgur.com/VJDzFAh.png)\n\n# Example terminal table\n![Imgur Image](https://imgur.com/ZncrWFX.png)\n\n# Example JSON object\n![Imgur Image](https://imgur.com/SfoDXHQ.png)\n",
    'author': 'fitzy1293',
    'author_email': 'berkshiremind@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Fitzy1293/redditsfinder',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
